%
% compute average densities and morphological data for a collection
% of pointclouds.
%
% subdir - atlas directory underwhich pointclouds.pce lives
% genetopc - table of pointclouds in that directory
% vis - whether to visualize maps as they are computed
%
% TODO: currently this function doesn't scale well.  it computes
% density maps for all the embryos in the database before averaging.
% should just compute weighted running totals instead
%
% charless fowlkes (c) 2007
%

function [dmap,vdmap,dcount,stats] = average_densities(subdir,genetopc,vis)

define_constants;

ext = '15'; %use the 15um radius density values
res = 200;  %compute map at 200x200 resolution

fprintf('computing density maps\n');
D = dir([subdir 'pointclouds.pce/*.pce']);
unz = zeros(res,res,length(D));
udensity = zeros(res,res,length(D));
tic
for i = 1:length(D)
  pcfile = [subdir 'pointclouds.pce/' D(i).name(1:end-4) '.pce'];
  fprintf('\r %d / %d    ',i,length(D));
  pc = load_pc(pcfile);
  [tX,tN] = transform(pc);
  %stats.egglength(i) = max(pc.X(:,1))-min(pc.X(:,1)); ... this doesn't work since the pointcloud is unaligned...
  stats.egglength(i) = 1./pc.scale(1,1);
  stats.nuclear_count(i) = pc.nuclear_count;
  stats.surfacearea(i) = sum(sarea(pc));
  stats.cnum(i) = getcohorts(pc);

  %compute angle between surface normal and microscope axis
  % since density is more reliable on surfaces perpundicular
  % to [0 0 1]
  unz(:,:,i) = render_signal(tX,abs(pc.N*[0 0 1]'));

  %unroll density estimated for this embryo
  udensity(:,:,i) = render_signal(tX,pc.data(:,getdensitychannel(pc,ext)));

  if (vis)
    figure(1); imagesc(udensity(:,:,i)); axis image; drawnow;
  end
end
tt = toc;
fprintf('%3.3f seconds for %d embryos = %3.3f seconds/embryo\n',tt,i,(tt/i));


exclude = find(stats.surfacearea>10^6); %exclude bad meshes

fprintf('averaging density maps\n');
dmap = zeros(res,res,numcohorts);
vdmap = zeros(res,res,numcohorts);
dcount = zeros(1,numcohorts);
for cht = 1:size(genetopc,2)
  select  = [];
  for gg = 1:size(genetopc,1)
    select = [select genetopc{gg,cht,MRNA}];
    select = [select genetopc{gg,cht,PROTEIN}];
  end
  select = setdiff(unique(select),exclude);
  dcount(cht) = length(select);
  %perform weighted averaging using unz as the weight.
  [dmap(:,:,cht),vdmap(:,:,cht)] = averagemaps(select,udensity,unz);
end;


fprintf('---------------------------\n');
for cht = 1:numcohorts
  select = setdiff(find(stats.cnum==cht),exclude);
  ave_egglength(cht) = mean(stats.egglength(select));
  std_egglength(cht) = std(stats.egglength(select));
  ave_surfacearea(cht) = mean(stats.surfacearea(select));
  std_surfacearea(cht) = std(stats.surfacearea(select));
  ave_nuclear_count(cht) = mean(stats.nuclear_count(select));
  std_nuclear_count(cht) = std(stats.nuclear_count(select));
  n = length(select);
  sc = (1.96/sqrt(n));
  fprintf('%s ',cohort_names{cht});
  fprintf('[%d] : ',n);
  fprintf('nucleii=%3.1f+/-%1.0f (sd=%2.2f),  ',ave_nuclear_count(cht),sc*std_nuclear_count(cht),std_nuclear_count(cht));
  fprintf('egglen=%3.1f+/-%1.0f (sd=%2.2f),  ',ave_egglength(cht),sc*std_egglength(cht),std_egglength(cht));
  fprintf('area=%3.1f+/-%1.0f (sd=%2.2f)\n  ',ave_surfacearea(cht),sc*std_surfacearea(cht),std_surfacearea(cht));
end
fprintf('---------------------------\n');
%select = find((stats.cnum>=4)&(stats.cnum<=9));
%ave_egglength = mean(stats.egglength(select));
%std_egglength = std(stats.egglength(select));
%ave_surfacearea = mean(stats.surfacearea(select));
%std_surfacearea = std(stats.surfacearea(select));
%ave_nuclear_count = mean(stats.nuclear_count(select));
%std_nuclear_count = std(stats.nuclear_count(select));
%n = length(select);
%sc = (1.96/sqrt(n));
%fprintf('[%d] : ',n);
%fprintf('nucleii=%3.1f+/-%1.0f (sd=%2.2f),  ',ave_nuclear_count,sc*std_nuclear_count,std_nuclear_count);
%fprintf('egglen=%3.1f+/-%1.0f (sd=%2.2f),  ',ave_egglength,sc*std_egglength,std_egglength);
%fprintf('area=%3.1f+/-%1.0f (sd=%2.2f)\n  ',ave_surfacearea,sc*std_surfacearea,std_surfacearea);
%fprintf('---------------------------\n');
%
%
%
%
%%
%for g = 1:size(genetopc,1)
%  fprintf('%s:',gene_names{g});
%  for c = 4:9
%    fprintf('%d ',size(genetopc{g,c,1},2));
%  end
%  fprintf('\n');
%end





