%
% this function scans a collection of pointclouds, builds a
% list of the genes they contain, and saves the results to
% a .mat file for later use
%
% subdir : the base directory where the atlas is being constructed
% subdir/pointclouds.pce/ should contain the collection of pointcloud
% files downloaded form the database.
%
% gene_names = list of gene names
% numgenes = number of genes
% genetopc = map from (gene,cohort,type) -> list of pointclouds
% hybs = list of hyb ids
% numhybs = number of distinct hybs
% genetohybind = map from (gene,type) -> index(s) into hybs.
%
%
% charless fowlkes (c) 2007
%

function [gene_names,numgenes,genetopc,hybs,numhybs,genetohybind,dyelist] = collect_metadata(subdir,genotype)

define_constants;

D =  dir([subdir 'pointclouds.pce/*.pce']);
fprintf('found %d pointcloud files in %s\n',length(D),subdir);

%first get the list of gene names
gene_names = {};
hybs = [];
for i = 1:length(D)
 fprintf('\r scanning pointcloud headers %d / %d ',i,length(D));
 pcfile = [subdir 'pointclouds.pce/' D(i).name(1:end-4) '.pce'];
 header = load_pc_header(pcfile);
 if (header.genotype_id == genotype)
   for j = 1:size(header.column_info,1)
     gene_names = union(gene_names,header.column_info{j,3});
   end
   hybs = [hybs; header.hybridization_id];
 end
end
gene_names = setdiff(gene_names,{'DNA','density','failed'});
[val,ind] = sort(lower(gene_names)); %sort them in a case insensitive manner
gene_names = gene_names(ind);
numgenes = length(gene_names);
fprintf('found %d unique genes\n',numgenes);

hybs = unique(hybs);
numhybs = length(hybs);
fprintf('found %d unique hybridizations\n',numhybs);


% now go back and build a lookup table of pointclouds
% for each (gene,cohort,type) triple
genetopc = cell(numgenes,numcohorts,numchanneltypes);
% also build a map from gene to the hybridizations containing that gene
genetohybind = cell(numgenes,numchanneltypes);
dyelist = zeros(length(D),numgenes);
for i = 1:length(D)
 fprintf('\r building pc table %d / %d ',i,length(D));
 pcfile = [subdir 'pointclouds.pce/' D(i).name(1:end-4) '.pce'];
 header = load_pc_header(pcfile);
 if (header.genotype_id == genotype)
    cohort = getcohorts(header);
    if (cohort < 11)
      [col,info,zerocount] = getgenechannels(header,gene_names);
      for j = 1:size(info,2)
        dye = dict(header.column_info{info(j),1},dye_names);
        type = dict(header.column_info{info(j),2},channeltype_names);
        gene = dict(header.column_info{info(j),3},gene_names);
        if (~isempty(type) & ~isempty(gene) & ~isempty(cohort))
          genetopc{gene,cohort,type} = [genetopc{gene,cohort,type},i];
        end
        genetohybind{gene,type} = union(genetohybind{gene,type},find(hybs==header.hybridization_id));
        dyelist(i,gene) = dye;
      end;
    else
      fprintf('skipping unstaged embryo\n');
    end;
  else
    fprintf('ERROR: ignoring pointcloud %s which does not match genotype\n',pcfile);
  end;
end
fprintf('\n');






%
%
%

%define_constants;
%genotype = MEL;
%D =  dir([subdir 'pointclouds.pce/*.pce']);
%fprintf('found %d pointcloud files in %s\n',length(D),subdir);
%
%for i = 1:length(D)
% fprintf('\r scanning pointcloud headers %d / %d ',i,length(D));
% pcfile = [subdir 'pointclouds.pce/' D(i).name(1:end-4) '.pce'];
% header = load_pc_header(pcfile);
% if (header.genotype_id == genotype)
%   for j = 1:size(header.column_info,1)
%     a = dict(header.column_info{j,3},genelist);
%     if ~isempty(a)
%       gg(i,j) =a;
%     end
%   end
% end
%end
%
