%
% this function maps between a list of strings and the string indices
% if you pass it a number (or array), it returns the string(s)
% if you pass it a string (or cellarray), it returns the number(s)
%
% unless nv contains undefined keys, this function satisfies
% dict(dict(nv,table)) = nv
%
% charless fowlkes (c) 2007
%

function nvmatch = dict(nv,table)  

if iscell(nv)
  nvmatch = zeros(1,length(nv));
  for i = 1:length(nv)
    match = strmatch(nv{i},table,'exact');
    if ~isempty(match)
      nvmatch(i) = match;
    else
      %error(sprintf('unable to find key ''%s'' in dict',nv{i}));
      sprintf('warning: unable to find key ''%s'' in dict',nv{i});
      nvmatch(i) = -1;
    end
  end;
elseif isstr(nv)
  nvmatch = strmatch(nv,table,'exact');
else
  if (length(nv)>1)
    nvmatch = cell(1,length(nv));
    for i = 1:length(nv)
      if ((nv(i) < 1) | (nv(i)>length(table)))
        error(sprintf('key value %d is out of range',nv(i)));
      end
      nvmatch{i} = table{nv(i)};
    end
  else
    nvmatch = table{nv};
  end
end

