%
% extract a/p boundaries of stripes based on template
%
% charless fowlkes, (c) 2007
%

function [matches,ematches,cost] = dp_register(tX,E,tplate,slices,mask,theta,vis)

if (nargin < 6)
  vis = false;
end;

% rotate by amount rot around the a/p axis
rot = [1 0 0; 0 cos(theta) sin(theta); 0 -sin(theta) cos(theta)];
tX = tX*rot;

pad = 10;       %pad boundary for edge extraction
E = E - prctile(E,0.5); % make sure signal is normalized
E = E / prctile(E,99.5);
[J,cx,cy] = render_signal(tX,E,200,'linear');
I = [J((201-pad):200,:); J; J(1:pad,:)];
df{1} = oeFilter(2*[3 1],3,pi/2,1);

fim = fbRun(df,I);
A = nonmax(fim{1},(pi/2))-nonmax(-fim{1},(pi/2));
A = A(pad+1:end-pad,:);
A = A.*(abs(A)>0);
A = A.*mask;

if (vis)
  figure(1); clf; imagesc(sign(A)+J.*mask); axis image; 
end;

map = A(slices,:);

edges = cell(size(map,1),1);
if vis, figure(1); hold on; end;
for k = 1:size(map,1)
  [ii,peaks,edges{k}.grad] = find(map(k,:));
  [val,edges{k}.nuc] = min(dist2([cx cy],[peaks' repmat(slices(k),length(peaks),1)]));
  edges{k}.peaks = peaks/200;
  if (vis)
    plot(peaks,slices(k)*ones(size(peaks)),'ko'); %actual peaks
    plot(cx(edges{k}.nuc),cy(edges{k}.nuc),'kx'); %nearest nucleus
  end;
end;

%
% now solve the k dynamic programs
%
if vis, figure(1); hold on; end;
for k = 1:length(edges)
  e1 = [0 edges{k}.grad 0]; x1 = [0 edges{k}.peaks 1];
  e2 = [0 tplate{k}.grad 0]; x2 = [0 tplate{k}.peaks 1];

  alpha = 10;
  beta = 0.5;
  m = length(e1); n = length(e2);
  C = zeros(m,n);
  Iinc = zeros(m,n); Jinc = zeros(m,n);
  for i = 2:m %loop over target
    for j = 2:n %loop over template
      mc = alpha*abs(x1(i)-x2(j)) + beta*abs( (sign(e1(i))-sign(e2(j))).*sign(e1(i)).*sign(e2(j)));
      match = C(i-1,j-1) + mc;
      skip_target = C(i,j-1) + 1;
      skip_template = C(i-1,j) + 0.5;
      [C(i,j),bp] = min([match,skip_target,skip_template]);
      switch bp
        case 1
          Iinc(i,j) = 1;
          Jinc(i,j) = 1;
        case 2
          Jinc(i,j) = 1;
        case 3
          Iinc(i,j) = 1;
      end; 
    end;
  end;

  T = zeros(m,n); T(m,n) = 1;
  i = m; j = n; 
  while ((i>1) & (j>1))
    in = i - Iinc(i,j);
    jn = j - Jinc(i,j);
    i = in; j = jn;
    T(i,j) = 1;
  end;

  for i = 2:n-1
    mt = find(T(:,i));
    if ~isempty(mt) & (mt(1)>1) %drop matches to boundary points
      matches{k}(i-1) = edges{k}.nuc(mt(1)-1);  %exclude 0 and 1
      ematches{k}(i-1) = edges{k}.peaks(mt(1)-1);  %exclude 0 and 1
    else
      matches{k}(i-1) = nan;
      ematches{k}(i-1) = nan;
    end;
    if (vis)
      if ~isnan(matches{k}(i-1))
        plot(200*[x1(mt(1)) x2(i)],[slices(k) slices(k)],'k-x','LineWidth',2);
      else
        plot(200*x2(i),slices(k),'r-o','LineWidth',2);
      end
    end
  end
end;
if vis, hold off; drawnow; end;

%
% construct list of matches
%
ex = []; et = [];
tx = []; tt = [];
for k = 1:length(matches)
  ind = find(~isnan(matches{k}));
  ex = [ex;ematches{k}(ind)'];
  et = [et;repmat(slices(k),length(ind),1)];
  tx = [tx;tplate{k}.peaks(ind)'];
  tt = [tt;repmat(slices(k),length(ind),1)];
end;

%
%compute quadratic cost associated with this matching
%
P1 = [ex et];
P2 = [tx tt];
vec = (P1-P2);
D = dist2(P1,P1);
Q = dist2(vec,vec) ./ (D + (D==0));
ind = find(sum(Q)<1);
nout = sum(sum(Q)>=1);
cost = sum(sum(abs(vec(ind,:))));

