%
% standardized cohort identifiers.  pass a pointcloud header to
% get the cohort.  alternately, pass no arguments to get the list
% of all cohorts
%
%%%%%%%%%%%%%%%%%%%%
%
%  from: clluengo@lbl.gov
%  date: feb 27, 2005
%
%   Yeah, you're right. Let's do that then:
%  
%   cohort 1:   4b
%   cohort 2:   4c
%   cohort 3:   4d
%   cohort 4:   5:0-3% 
%   cohort 5:   5:4-8% 
%   cohort 6:   5:9-25% 
%   cohort 7:   5:26-50%
%   cohort 8:   5:51-75% 
%   cohort 9:   5:76-100% 
%   cohort 10:  6
%  
%   This is the last time I want to hear anything about cohorts. Capice?
%  

%
% charless fowlkes (c) 2007
%

function [cohort,names] = getcohorots(PC)

cohort = [];

% cohorts without stages are generally too crappy to use, 
% so don't touch any except the mutants where staging is
% necessarily somewhat uncertain.
uncertain_stage = {
  's10772-08au05-05', 0,200
  's10848-03oc05-05', 0,100
  's10848-03oc05-06', 0,100
  's10865-02ja06-10',40, 40
  's10865-02ja06-13',35, 40
  's10865-05de05-29',40, 40
  's10866-03ja06-06', 2, 20
  's10881-03ja06-07', 2, 10
  's10882-04ja06-28',35, 35
  's10882-04ja06-36',40, 45
  's10882-05de05-26',40, 45
  's10882-05de05-27', 7, 15
  's10883-04ja06-43', 2, 10
  's10883-06de05-04', 2, 10
  's10917-03de05-24',35, 40
  's10917-03de05-25', 2, 30
  's10917-03de05-27', 2, 30
  's10917-03de05-29', 2, 30
  's10917-03de05-30', 5, 15
  's10917-20de05-25', 2, 30
  's10917-20de05-26', 2, 20
  's10917-20de05-27', 2, 30
  's10917-21de05-29', 2, 30
  's10917-21de05-30', 5, 15
  's10917-28oc05-08', 0, 10
  's10918-05de05-01', 2, 30
  's10918-22de05-01', 2, 30
  's10918-22de05-02',40, 40
  's10919-09ja06-19',10, 35
  's10919-28oc05-13',10, 35
};

for j = 1:size(uncertain_stage,1)
  uc.name{j} = uncertain_stage{j,1};
  uc.lb(j) = uncertain_stage{j,2};
  uc.ub(j) = uncertain_stage{j,3};
end;

if (nargin==1)
for i = 1:length(PC)
  stage = PC(i).stage;
  pct = PC(i).stage_percent;

  index = strmatch(PC(i).name,uc.name);
  if ~isempty(index)
    pct = 0.5*(uc.lb(index)+uc.ub(index)); 
  end;

  if strcmp(stage,'4b')  
    cohort(i) = 1;
  elseif strcmp(stage,'4c')  
    cohort(i) = 2;
  elseif strcmp(stage,'4d')  
    cohort(i) = 3;
  elseif strcmp(stage,'5')
    if ~isnan(pct)
      if ((pct >= 0) & (pct <= 3))
        cohort(i) = 4;
      elseif ((pct > 3) & (pct <= 8))
        cohort(i) = 5;
      elseif ((pct > 8) & (pct <= 25))
        cohort(i) = 6;
      elseif ((pct > 25) & (pct <= 50))
        cohort(i) = 7;
      elseif ((pct > 50) & (pct <= 75))
        cohort(i) = 8;
      elseif ((pct > 75) & (pct <= 100))
        cohort(i) = 9;
      end;
    else
      cohort(i) = 11; %stage 5 but unknown percent
    end;
  elseif strcmp(stage,'6a')
    cohort(i) = 10;
  elseif strcmp(stage,'undefined');
    cohort(i) = 11;
  else
    cohort(i) = 11;
  end;
end;
else 
  cohort = [];
end
names = {'4b','4c','4d','5:0-3','5:4-8','5:9-25','5:26-50','5:51-75','5:76-100','6a','undefined'};

