%
% extract the default expression channel column indicies 
%
% if a subcellular segmentation reports zero volume, default
% to the general value recorded for the entire cell.
%
% charless fowlkes (c) 2007
%

function [columnind,infoind,zerocount] = getgenechannels(pc,gene_names,usecy5)

%minimum expected volume, if it is below this we
% assume there was an error in the subcellular 
% localization
minvol = 5;

% whether to use the cy5 channels.
if (nargin < 3)
  usecy5 = false;
end;

zerocount = 0;
columnind = [];
infoind = [];
ct = 1;
for i = 1:size(pc.column_info,1)
  if ( (strcmp(pc.column_info{i,5},'Gene Expression') || strcmp(pc.column_info{i,5},'Gene Expression Average')) && (usecy5 || ~strcmp(pc.column_info{i,1},'Cy5')) )
    genenum = dict(pc.column_info{i,3},gene_names);
    if strcmp(pc.column_info{i,3},'failed')
      fprintf('warning: skipping "failed" stain');
      continue;
    end;
    if (isempty(genenum))
      error(sprintf('could not match %s to any known gene\n',pc.column_info{i,3}));
    end;
    if ~isempty(pc.column_info{i,6})
      columnname = [pc.column_info{i,1} '_' pc.column_info{i,6}];
      columnnamecell = [pc.column_info{i,1} '_cell'];
    else
      columnname = pc.column_info{i,1};
    end;
    %build mask in order to remove zero volume elements
    if isfield(pc,'data')
      if ~isempty(pc.column_info{i,6})
        volumename = ['V' pc.column_info{i,6}(1)];  % eg.  Vn,Va,Vb, or Vc
        volind = strmatch(volumename,pc.column,'exact');
        valid = (pc.data(:,volind)>minvol); %only use reasonably large volumes
        zerocount = max(zerocount,sum(1-valid));
      else
        %if prefered volume field is missing, e.g. in an atlas, assume all volumes are valid
        valid = ones(size(pc.data(:,1))); 
      end
      columnind(:,ct) = strmatch(columnname,pc.column,'exact').*valid + strmatch(columnnamecell,pc.column,'exact').*(1-valid);
    else
      columnind(:,ct) = strmatch(columnname,pc.column,'exact')*ones(pc.nuclear_count,1);
      %fprintf('warning: zero volume masking disabled\n');
    end

    infoind(ct) = i;
    ct = ct + 1;
  end;
end;

