%
% read in a correspondence file
%  see also: save_corr
%
% charless fowlkes, (c) 2007
%

function  corstruct = load_corr(filename)

corstruct = [];
if (~fileexists(filename))
  error(sprintf('error opening correspondence file %s',filename));
end;
fid = fopen(filename,'r');

%parse the header
L = fgetl(fid);
L = eatcomments(fid,L);
while (ischar(L) & findstr(L,'#'))
  L = eatprefix(L);
  [name,value] = parsenvpair(L); 
  corstruct = setfield(corstruct,name,value);
  L = fgetl(fid);
  L = eatcomments(fid,L);
end;
fclose(fid);

% now read in the cell data and build a sparse 
% matrix to represent the matching
D = dataread('file',filename,'','commentstyle','shell','delimiter',',');
ii = D(:,1);
nz = D(:,2);
ct = 1;
for i = 1:corstruct.dst_nuclearcount
  x(ct:ct+nz(i)-1) = ii(i)*ones(nz(i),1);
  y(ct:ct+nz(i)-1) = D(i,2+(1:nz(i)));
  z(ct:ct+nz(i)-1) = D(i,(2+nz(i))+(1:nz(i)));
  ct = ct + nz(i);
end
corstruct.match = sparse(x,y,z,corstruct.dst_nuclearcount,corstruct.src_nuclearcount);


%
% skip over lines starting with '##'
%

function L = eatcomments(fid,L)

while (ischar(L) & findstr(L,'##'))
  L = fgetl(fid);
end;

%
% eat leading # and whitespace
%

function L = eatprefix(L)
while(isspace(L(1)) | strcmp(L(1),'#'))
  L = L(2:end);
end;





