%
% load in the new pointcloud file format
%
% charless fowlkes (c) 2005-2007
%

function pcstruct = load_pc(filename)

pcstruct = [];
if (~fileexists(filename))
  error(sprintf('error opening pc file %s',filename));
end;
fid = fopen(filename,'r');

%parse the header
L = fgetl(fid);
L = eatcomments(fid,L);
while (ischar(L) & findstr(L,'#'))
  L = eatprefix(L);
  [name,value] = parsenvpair(L); 
  pcstruct = setfield(pcstruct,name,value);
  L = fgetl(fid);
  L = eatcomments(fid,L);
end;
fclose(fid);

if ~isfield(pcstruct,'column')
  error('no column field in pc file, parsing aborted');
end;

%remove geometry columns from header structure
ncolumns = length(pcstruct.column);
idind = strmatch('id',pcstruct.column);
coordind = [strmatch('x',pcstruct.column,'exact') strmatch('y',pcstruct.column,'exact') strmatch('z',pcstruct.column,'exact')];
normalind = [strmatch('Nx',pcstruct.column,'exact') strmatch('Ny',pcstruct.column,'exact') strmatch('Nz',pcstruct.column,'exact')];
remainder = setdiff(1:ncolumns,[idind coordind normalind]);
pcstruct.column = pcstruct.column(remainder);
if (isfield(pcstruct,'column_factors'))
  pcstruct.column_factors = pcstruct.column_factors(remainder);
end;


%combine column_info and column_info_bid
if (isfield(pcstruct,'column_info_bid'))
  len = size(pcstruct.column_info,1);
  for k = len+1:len+size(pcstruct.column_info_bid,1)
    for l = 1:size(pcstruct.column_info,2)
      pcstruct.column_info{k,l} = pcstruct.column_info_bid{k-len,l};
    end;
  end;
  pcstruct = rmfield(pcstruct,'column_info_bid');
end;

%
% deal with transgenes
%
if strcmp(pcstruct.phenotype_string(1:8),'D. melan')
  pcstruct.genotype_id = 1;
  [drop,tgstring] = strtok(pcstruct.phenotype_string,',');
  tgstring = strtok(tgstring,',');
  tgstring = tgstring(2:end);
  for k = 1:size(pcstruct.column_info,1)
    if strcmp(pcstruct.column_info{k,3},'LacZ')
      pcstruct.column_info{k,3} = ['LacZ:' tgstring];
      pcstruct.column_info{k,4} = ['LacZ:' tgstring];
      pcstruct.column_info{k,6} = 'nuclear';
    end
  end
end

%
% skip over lines starting with '##'
%

function L = eatcomments(fid,L)

while (ischar(L) & findstr(L,'##'))
  L = fgetl(fid);
end;

%
% eat leading # and whitespace
%

function L = eatprefix(L)
while(isspace(L(1)) | strcmp(L(1),'#'))
  L = L(2:end);
end;





