%
% parse a name value pair in the pointcloud file
%
% charless fowlkes (c) 2007
%
function [name,value] = parsenvpair(nvstring)

  [name,value] = mystrtok(nvstring,'=');
  name = eatwhite(name);
  value = eatwhite(value);
  if isempty(value)
     value = [];
     if isempty(name)
       error(sprintf('empty name and value?? %s = %s',name,value));
     end;
  elseif (value(1) == '"')  %a string
    if (value(end) ~= '"')
      error(sprintf('malformed string %s = %s',name,value));
    else
      value = value(2:end-1);
    end;
  elseif (value(1) == '[') %an array
    remainder = eatwhite(eatbrackets(value));
    if isempty(remainder)
      value = [];
    else
      if (remainder(1) == '"') %string array
        value = cell(0); 
        i = 1;
        [line,remainder] = mystrtok(remainder,';');
        while ~isempty(remainder)
          j = 1;
          [entry,line] = mystrtok(line,',');
          while ~isempty(line)
            value{i,j} = eatquotes(eatwhite(entry));
            [entry,line] = mystrtok(line,',');
            j = j + 1;
          end;
          value{i,j} = eatquotes(eatwhite(entry));
          [line,remainder] = mystrtok(remainder,';');
          i = i + 1;
        end;
        %eat the last line
        j = 1;
        [entry,line] = mystrtok(line,',');
        while ~isempty(line)
          value{i,j} = eatquotes(eatwhite(entry));
          [entry,line] = mystrtok(line,',');
          j = j + 1;
        end;
        value{i,j} = eatquotes(eatwhite(entry));
      else %numerical array
        value = str2num(value);
      end;
    end;
  else %a number
    value = str2num(value);
  end;

%
% eat leading and trailing whitespace
%

function L = eatwhite(L)
while (length(L)>0)&isspace(L(1))
  L = L(2:end);
end;
while (length(L)>0)&isspace(L(end))
  L = L(1:end-1);
end;


%
% eat leading and trailing double quotes 
%

function L = eatquotes(L)
while (length(L)>0)&(L(1)=='"')
  L = L(2:end);
end;
while (length(L)>0)&(L(end)=='"')
  L = L(1:end-1);
end;


%
% eat leading and trailing brackets 
%

function L = eatbrackets(L)
while (length(L)>0)&(L(1)=='[')
  L = L(2:end);
end;
while (length(L)>0)&(L(end)==']')
  L = L(1:end-1);
end;

