%
% render a thumbnail of the unrolled expression pattern
% for a single channel
%
% will take either pre-transformed geometry (Nx3 array) or pcstruct
%
% image contains the resulting image
% cx,cy contain the coordinates of the nucleii with respect to the image
%
% charless fowlkes (c) 2005-2007
%

function [image,cx,cy] = render_signal(pc,signal,resolution,interp)

if (nargin < 4)
  interp = 'linear';
end;
if (nargin < 3)
  resolution = 200;
end;

if isstruct(pc)
  Xr = transform(pc);
  Xc = Xr(:,1);
  Yc = (mod(atan2(Xr(:,2),Xr(:,3)),2*pi))/(2*pi);
else
  %assume already transformed etc.
  Xc = pc(:,1);
  Yc = (mod(atan2(pc(:,2),pc(:,3)),2*pi))/(2*pi);
end;

%pad out periodic boundary condition
Xc1 = [Xc;Xc;Xc];
Yc1 = [Yc-1;Yc;Yc+1];
if (size(signal,1)==1)
  signal1 = [signal'; signal'; signal'];
else
  signal1 = [signal; signal; signal];
end;
%this should be plenty conservative, 
%usually adds ~700 points
good = find((Yc1>-0.05) & (Yc1<1.05)); 
Xc1 = Xc1(good);
Yc1 = Yc1(good);
signal1 = signal1(good);

%
% now do the interpolation
%
[xx,yy] = meshgrid(linspace(0,1,resolution),linspace(0,1,resolution));
image = griddata(Xc1,Yc1,signal1,xx,yy,interp);
image2 = griddata(Xc1,Yc1,signal1,xx,yy,'nearest');
image(isnan(image)) = image2(isnan(image));

cx = resolution*Xc;
cy = resolution*Yc;

