%
% estimate the surface area assocated with every vertex
%  see: meyer etal, "discrete differental-geometry operators ..."
%
% charless fowlkes, (c) 2007

function area = sarea(pc)

area = zeros(pc.nuclear_count,1);

fprintf('computing surface area [');
for i = 1:pc.nuclear_count
  %compute the surface area associated with each vertex
  area(i) = 0;
  p = pc.X(i,:);
  for k = 1:pc.nnbrs(i)
    q = pc.X(pc.nbrs(i,k),:);
    r = pc.X(pc.nbrs(i,mod(k,pc.nnbrs(i))+1),:);
    area(i) = area(i) + subarea(p,q,r);
  end;
  if (mod(i,100)==1)
    fprintf('.');
  end;
end;
fprintf('.]\n');

%
% compute the area of the "voronoi region" about p of the triangle pqr.
%
function area = subarea(p,q,r)

  %compute the three angles of the triangle
  pq = p-q;
  pr = p-r;
  qr = q-r;
  tp = acos((pq*pr')./(norm(pq)*norm(pr)));
  tq = acos((-pq*qr')./(norm(pq)*norm(qr)));
  tr = acos((pr*qr')./(norm(pr)*norm(qr)));

  if (tp>=(pi/2))  
    %obtuse at point p
    s = 0.5*(norm(pq)+norm(pr)+norm(qr));
    tarea  = sqrt(s*(s-norm(pq))*(s-norm(pr))*(s-norm(qr)));
    area = (1/2)*tarea;
  elseif (tq>=(pi/2)) | (tr>=(pi/2)) 
    %obtuse at q or r 
    s = 0.5*(norm(pq)+norm(pr)+norm(qr));
    tarea  = sqrt(s*(s-norm(pq))*(s-norm(pr))*(s-norm(qr)));
    area = (1/4)*tarea;
  else 
    %not obtuse
    area = (1/8) * (norm(pr)^2*cot(tq) + norm(pq)^2*cot(tr));
  end;








%figure(1); hold on;
%plot([p(1) q(1)],[p(2) q(2)],'r-','LineWidth',3),;
%plot([p(1) r(1)],[p(2) r(2)],'g-','LineWidth',3),;
%plot([q(1) r(1)],[q(2) r(2)],'b-','LineWidth',3),;
%c = mean([p;q;r]);
%c = c + randn(size(c));
%plot(c(1),c(2),'bx')

%fprintf('computing surface area [');
%for i = 1:pc.nuclear_count
%  %compute the surface area
%  area(i) = 0;
%  p0 = pc.X(i,:);
%  p1 = pc.X(pc.nbrs(i,pc.nnbrs(i)),:);
%  p2 = pc.X(pc.nbrs(i,1),:);
%  p3 = pc.X(pc.nbrs(i,2),:);
%  vec1 = p0 - p1; vec2 = p2 - p1; alpha = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%  vec1 = p0 - p3; vec2 = p2 - p3; beta = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%  area(i) = area(i) + (1/8)*(cot(alpha)+cot(beta))*(norm(p2-p0)^2);
%  for k = 2:pc.nnbrs(i)-1
%    p1 = pc.X(pc.nbrs(i,k-1),:);
%    p2 = pc.X(pc.nbrs(i,k),:);
%    p3 = pc.X(pc.nbrs(i,k+1),:);
%    vec1 = p0 - p1; vec2 = p2 - p1; alpha = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%    vec1 = p0 - p3; vec2 = p2 - p3; beta = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%    area(i) = area(i) + (1/8)*(cot(alpha)+cot(beta))*(norm(p2-p0)^2);
%  end;
%  p1 = pc.X(pc.nbrs(i,pc.nnbrs(i)-1),:);
%  p2 = pc.X(pc.nbrs(i,pc.nnbrs(i)),:);
%  p3 = pc.X(pc.nbrs(i,1),:);
%  vec1 = p0 - p1; vec2 = p2 - p1; alpha = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%  vec1 = p0 - p3; vec2 = p2 - p3; beta = acos((vec1*vec2')./(norm(vec1)*norm(vec2)));
%  area(i) = area(i) + (1/8)*(cot(alpha)+cot(beta))*(norm(p2-p0)^2);
%  if (mod(i,100)==1)
%    fprintf('.');
%  end;
%end;
%fprintf('.]\n');
%
% test codes
%
%p = [0 0]
%q = [0 0.8];
%r = [.51 .51];
%
%figure(1); clf; hold on
%plot(p0(1),p0(2),'ro','LineWidth',3),;
%plot(p1(1),p1(2),'go','LineWidth',3),;
%plot(p2(1),p2(2),'bo','LineWidth',3),;
%plot(p3(1),p3(2),'ko','LineWidth',3),;
%axis square; axis([-0.1 1.1 -0.1 1.1]);

