%
% generate a time-series atlas file 
%
% X = cell array of coordinates over time, each entry is (numcells x 3)
% Ex = expression over time, each entry is (numcells x numgenes)
% nnbrs = neighborhood relation between cells (numcells x 1)
% nbrs = neighborhood relation between cells (numcells x max(nnbrs))
%    (TODO: assumed static for now but clearly shouldn't be)
% genenames = cell arary of short gene names
% types = whether each column is mRNA, protein, density, etc.
% cohort_names = cell array of cohort names (e.g. '5:0-3')
% cohort_times = cell array of cohort times (in minutes)
% embryospergene = number of embryos averaged for each Ex entryo (numcells x numgenes)
% srclist = list of pointclouds averaged into this virtual embryo [[optional]]
%
% charless fowlkes (c) 2007
%

function save_atlas(filename,X,Ex,nnbrs,nbrs,genenames,types,cohort_names,cohort_times,embryospergene,srclist)

fid = fopen(filename,'w');


%TODO: some other error checking here?
ntime = length(X);
assert(length(Ex)==ntime);
assert(length(cohort_names)==ntime);
assert(length(cohort_times)==ntime);
assert(size(embryospergene,1)==ntime);

ncells = size(X{1},1);
assert(length(nnbrs)==ncells);
assert(size(nbrs,1)==ncells);

ngenes = length(genenames)
assert(length(types)==ngenes);
assert(size(embryospergene,2)==ngenes);


%prune out empty cohorts, pcx doesn't seem to like them
% TODO: fix pcx
goodcohorts = find(sum(embryospergene,2)>0);
%ntime = length(goodcohorts);
%X = X(goodcohorts);
%Ex = Ex(goodcohorts);
%embryospergene = embryospergene(goodcohorts,:);
% But, BID upload expects full list of cohortnames in order to correlate 
% with thumbnails so don't prune that list...
%cohort_names = cohort_names(goodcohorts);
%cohort_times = cohort_times(goodcohorts);

fprintf(fid,'##\n');
fprintf(fid,'## atlas created: %s\n',date);
fprintf(fid,'##\n');
fprintf(fid,'# cohort_names = ['); fprintf(fid,'"%s",',cohort_names{1:end-1}); fprintf(fid,'"%s"]\n',cohort_names{end});
fprintf(fid,'# cohort_times = ['); fprintf(fid,'%d,',cohort_times(1:end-1)); fprintf(fid,'%d]\n',cohort_times(end));
if ~isempty(srclist)
  fprintf(fid,'# srclist = ['); fprintf(fid,'"%s",',srclist{1:end-1}); fprintf(fid,'"%s"]\n',srclist{end});
end;

use = find(sum(embryospergene,1)>0);
fprintf(fid,'# embryospergene = [');
for t = 1:ntime-1
  fprintf(fid, '%d,',embryospergene(t,use(1:end-1))); fprintf(fid,'%d;',embryospergene(t,use(end)));
end
fprintf(fid, '%d,',embryospergene(ntime,use(1:end-1))); fprintf(fid,'%d]\n',embryospergene(ntime,use(end)));
fprintf(fid,'##\n');
fprintf(fid,'## Column Info\n');
fprintf(fid,'##\n');
fprintf(fid,'# column_info = [');

% only include descriptors of genes that are
% used at least once
use = find(sum(embryospergene,1)>0);
for i = 1:length(use)
  if strcmp(types{use(i)},'mRNA')
    fprintf(fid,'"%s","mRNA","%s","%s","Gene Expression Average",""',genenames{use(i)},genenames{use(i)},genenames{use(i)});
  elseif strcmp(types{use(i)},'protein')
    fprintf(fid,'"%sP","protein","%s","%s protein","Gene Expression Average",""',genenames{use(i)},genenames{use(i)},genenames{use(i)});
  elseif strcmp(types{use(i)},'density')
    fprintf(fid,'"density","","density","Nuclear Density","Derived Morphology",""');
  elseif strcmp(types{use(i)},'matchdist')
    fprintf(fid,'"%s","","matchdist","%s","Expression Difference",""',genenames{use(i)});
  else
    fprintf('%s',types{use(i)});
    error('unknown type');
  end;
  if (i < length(use))
    fprintf(fid,'; ');
  else
    fprintf(fid,']\n');
  end;
end;
fprintf(fid,'##\n');

%fprintf(fid,'# column = ["id","x","y","z","Nx","Ny","Nz","Da","Db","Vn","Vc",');
fprintf(fid,'# column = ["id"');
for t = 1:ntime
  use = find(embryospergene(t,:)>0); % only write out non-empty columns
  fprintf('writing %d genes at time step %d\n',length(use),t);
  fprintf(fid,',"x__%d","y__%d","z__%d","Nx__%d","Ny__%d","Nz__%d"',t,t,t,t,t,t);
  for i = 1:length(use)
    if strcmp(types{use(i)},'mRNA')
      fprintf(fid,',"%s__%d"',genenames{use(i)},t);
    elseif strcmp(types{use(i)},'protein')
      fprintf(fid,',"%sP__%d"',genenames{use(i)},t);
    elseif strcmp(types{use(i)},'density')
      fprintf(fid,',"density__%d"',t);
    elseif strcmp(types{use(i)},'matchdist')
      fprintf(fid,',"matchdist__%d"',t);
    else
      fprintf('%s',types{use(i)});
      error('unknown type');
    end;
  end;
end;
fprintf(fid,']\n');

fprintf('re-estimating normals');
for t = 1:ntime
  fprintf('.');
  N{t} = normals(X{t},nbrs,nnbrs); 
end;

fprintf(fid,'##\n');
fprintf(fid,'## Nuclei Info\n');
fprintf(fid,'##\n');
fprintf(fid,'# nuclear_count = %d\n',size(X{1},1));
fprintf(fid,'##\n');
for i = 1:size(X{1},1)
  fprintf(fid,'%d,',i);
  for t = 1:ntime
    fprintf(fid,'%.6f,%.6f,%.6f,',X{t}(i,1),X{t}(i,2),X{t}(i,3));
    fprintf(fid,'%.6f,%.6f,%.6f,',N{t}(i,1),N{t}(i,2),N{t}(i,3));
    use = find(embryospergene(t,:)>0); % only write out non-empty columns
    if ~isempty(use)
      fprintf(fid,'%.6f,',Ex{t}(i,use));
    end
  end;
  fprintf(fid,'%d,',nnbrs(i));
  for j = 1:nnbrs(i)-1
    fprintf(fid,'%d,',nbrs(i,j));
  end;
  fprintf(fid,'%d\n',nbrs(i,nnbrs(i)));
end;





