%
% write out a correspondence between two pointclouds
% 
% filename - name of file to write to
% 
% match - a matrix whose dimension are Ndestnuclei x Nsourcenuclei
%    there is a row for destination nucleus and nonzero entries in 
%    that row indicate nuclei from the source embryo which correpsond
%    to that destination nucleus.  entries may take on different values
%    in order to specify relative weightings in a "soft matching"
%
% dstname - name of destination onto which src is being registered
%
% srcname - source pointcloud name
%
% tdist - total deformation displacement
%
% qcost - quadratic matching cost
%
% failed - flag = true if matching process failed.  indicates the registration
%    resorted to coarse alignment
%
% charless fowlkes, (c) 2007
%

function  save_corr(filename,match,dstname,srcname,tdist,qcost,failed)

fid = fopen(filename,'w');

fprintf(fid,'## correspondence file - written %s\n',datestr(now));
fprintf(fid,'##\n');
fprintf(fid,'## format is: dst_id,nmatches,src_id_1,src_id_2,...,weight_1,weight_2, ....');
fprintf(fid,'##\n');
fprintf(fid,'##\n');
fprintf(fid,'# src_name="%s"\n',srcname);
fprintf(fid,'# src_nuclearcount=%d\n',size(match,2));
fprintf(fid,'# dst_name="%s"\n',dstname);
fprintf(fid,'# dst_nuclearcount=%d\n',size(match,1));
fprintf(fid,'# total_displacement=%3.6f\n',tdist);
fprintf(fid,'# distortion_cost=%3.6f\n',qcost);
fprintf(fid,'# matching_failed=%d\n',failed);
fprintf(fid,'##\n');
for i = 1:size(match,1)
  ind = find(match(i,:));
  nnz = length(ind);
  if (nnz > 0)
    fprintf(fid,'%d,%d,',i,nnz);
    fprintf(fid,'%d,',ind);
    if (nnz > 1)
      fprintf(fid,'%3.6f,',full(match(i,ind(1:end-1))) );
    end
    fprintf(fid,'%3.6f\n',full(match(i,ind(end))) );
  else
    fprintf(fid,'0\n');
  end;
end;
fclose(fid);


