
%
% perform gaussian process regression on the hybridization 
% max values in order to estimate a timecourse with which
% to scale all expression measurements
%
% if a given gene doesn't have any hybridizations which span 
% more than 1 cohort then just default to no gain scaling.  
%
% charless fowlkes (c) 2007
%

function gainfit = smoothgains(hybmaxave,hybmaxavect,uhlist,ct,numgenes,gene_names,vis)
define_constants;
addpath gpml; %add the gpr toolbox
gainfit = zeros(numgenes,numcohorts,numchanneltypes);
timerange = squeeze(sum(ct,2)>1); %genes with more than 1 time point
if (size(timerange,2)==1)
  fprintf('WARNING: only 1 gene detected\n');
  timerange = timerange';  %only a single gene!!
end
for tt = 1:numchanneltypes
  for gg = 1:numgenes
    fprintf('\r fitting gains to %s,%s   ',gene_names{gg},channeltype_names{tt});
    % see if we have data to do the fit
    if ~timerange(gg,tt) 
      gainfit(gg,:,tt) = 0; %default to unit gain
      continue;
    end
    goodhybs = unique([uhlist{gg,:,tt}]);
    numreps = length(goodhybs); %number of replications of a hybridization
    if (numreps==0)
      gainfit(gg,:,tt) = 0; %default to unit gain
      continue;
    end
    tseries = zeros(numcohorts,numreps);  %99th percentiles
    ctseries = zeros(numcohorts,numreps); %number of embryos
    for k = 1:numreps
      tseries(:,k) = hybmaxave(:,gg,goodhybs(k),tt);
      ctseries(:,k) = hybmaxavect(:,gg,goodhybs(k),tt);
    end

    %multiplicatively scale each timeseries so as to minimize the 
    %rms error between them
    % 
    % TODO: improve this optimization proceedure...currently
    % gets stuck in local minima
    %
    %tseriesorig = tseries;
    tseries(tseries==0) = NaN;
    opt = optimset('MaxFunEvals',1000000,'MaxIter',100000);
%    if (false)
%      [val,ind] = sort(-sum(ctseries));
%      for rd = 2:length(ind)
%        ind(1:rd)
%        lscale = log(nanmean(tseries));
%        lscale(ind(1:rd)) = fminsearch(@(x) nanrms(x,tseries(:,ind(1:rd))),lscale(ind(1:rd)),opt);
%        lscale
%        fprintf('----\n');
%        scale = exp(lscale);
%        tseries = tseries ./ repmat(scale,numcohorts,1);
%        %tseries = tseries ./ nanmedian(tseries(:));
%        subplot(3,2,rd-1); plot(tseries,'.-'); 
%      end
%      nanrms(zeros(size(lscale)),tseries)
%    else
      lscale = log(nanmean(tseries));
      lscale = fminsearch(@(x) nanrms(x,tseries),lscale,opt);
      scale = exp(lscale);
      tseries = tseries ./ repmat(scale,numcohorts,1);
      %plot(tseries,'.-');
%    end
    tseries = tseries ./ nanmedian(tseries(:));

    % replicate observations in proportion to the number of embryos
    xval = []; yval = [];
    for k = 1:numreps
      for cht = 1:numcohorts
        n = ctseries(cht,k);
        xval = [xval repmat(cohort_times(cht),1,n)];
        yval = [yval repmat(tseries(cht,k),1,n)];
      end;
    end;
    good = ~isnan(yval);
    xval = xval(good)';
    yval = yval(good)';

    covfunc = {'covSum', {'covSEiso','covNoise'}};
    loghyper = [log(30) log(sqrt(1.22)) -0.5]';
    yvalsmooth = gpr(loghyper,covfunc,xval,log(yval),cohort_times');
    gainfit(gg,:,tt) = exp(yvalsmooth');

    if (vis)
      xstar = linspace(min(cohort_times),max(cohort_times),201)';
      [mu,S2] = gpr(loghyper,covfunc,xval,log(yval),xstar);
      f = [mu+sqrt(S2);flipdim(mu-sqrt(S2),1)];
      if (gg<10)
        figure(1); subplot(3,3,gg);
      else
        figure(2); subplot(3,3,gg-9);
      end
      set(gca,'FontSize',25);
      hold on;
      fill([xstar; flipdim(xstar,1)], exp(f), [7 7 7]/8, 'EdgeColor', [7 7 7]/8);
      plot(xval, yval, 'ko','MarkerSize',2);
      plot(xstar,exp(mu),'k-','LineWidth',5);
      plot(cohort_times,tseries,'.-','LineWidth',2); 
      title(sprintf('%s',gene_names{gg}));
      axis square; axis tight; 
      axis([70 120 0 3]);
      drawnow; 
    end
  end
end


%if (vis)
%  for gg = 1:numgenes
%    figure(gg);
%    print(gcf,[subdir '/gainfit-' num2str(gg) '.eps'],'-depsc2');
%  end;
%end;
